if( exists("WAME.manual") ) {
	source("WAME.paired.R");
}

WAME.general<-function(X,design,contrasts,Sigmas,alpha,Sigma.stars,estimation.method="full nML",cache.dir=getOption("WAME.cache.dir")){
	#TODO: make verifyX
	#TODO: make verifyDesign
	#TODO: split by NA functionality
	#TODO: make verifyTestability

	#Produce the set of projection matrices, according to design and contrasts
	Ps<-WAME.projections(X=X,D=design,C=contrasts);

	#If Sigma.Z is given, then it can be used as Sigma.star.Z if we need it when estimating alpha
	if( !missing(Sigmas) ) {
		if( !is.list(Sigmas) ) Sigmas=list(Z=Sigmas);
		Sigma.stars<-Sigmas;
	}
	
	#If Sigma.stars is not given, then estimate it
	if( missing(Sigma.stars) ) {
		Sigma.stars<-estimateSigmaStar(X=X,estimation.method=estimation.method,Ps=Ps,cache.dir=cache.dir);
	} else {
		if( !is.list(Sigma.stars) ) Sigma.stars=list(Z=Sigma.stars);
	}
	
	#make the data to base the further analysis on
	Z<-X%*%t(Ps$x2z);
	designs=lapply(list(X=design, Y=Ps$x2y%*%design, Z=Ps$x2z%*%design),realZero);
	
	#If either Sigma or alpha are not given, estimate the scale, lambda, for
	# Sigma.star and alpha and form the estimate of Sigma.
	if( missing(Sigmas) || missing(alpha) ) {
		hyper.par<-estimateHyperPar(Z,designs$Z,Sigma.stars$Z);
		alpha<-hyper.par$alpha;
		Sigmas=lapply(Sigma.stars,function(ss){calcSigma(ss,hyper.par$lambda)});
	}
	
	#Calculate the coefficient-wise weights for the estimation.
	weights.Z=calcWeights(designs$Z,contrasts,Sigmas$Z);
	weights=list(X=weights.Z%*%Ps$x2z,Y=weights.Z%*%Ps$y2z,Z=weights.Z);
	
	#Calculate the degrees of freedom like parameters (F1, F2, (t,) n, k, p)
	df<-calcDf(designs$Z,contrasts,alpha);
	
	#Calculate the gene-wise estimates of the coefficients (i.e. 'contrasts' times the parameter vector)
	coefficients<-calcCoef(Z,weights$Z);
	
	#Calculate the gene-wise variance-related statistic S
	S<-calcS(Z,designs$Z,Sigmas$Z);
	
	#Perform the test, i.e. calculate the gene-wise statistics of interest, as well as the related p-values.
	stats<-calcStats(Z,designs$Z,contrasts,Sigmas$Z,coefficients,S,df);
	
	#Return the results compiled in a list
 	res<-append(stats, list(coefficients=coefficients, designs=designs, contrasts=contrasts, weights=weights, alpha=alpha, S=S, Sigmas=Sigmas, df=df, Ps=Ps));
	class(res)<-"WAME.general.result";
	return(res);
}