hash.fS<-list( #the hash functions making the fingerprint of the data
	dimension=function(X){
		dim(X);
	},
	array.wise.mean=function(X){
		colMeans(X);
	},
	array.wise.median=function(X){
		apply(X,2,median);
	},
	naive.covariance.estimate=function(X){
		cov(X);
	}
)

find.SigmaStar.in.cache<-function(X,p.name,cache.dir){
	if( is.null(cache.dir) || is.null(p.name) ) return(NULL);
	cache.file=paste(cache.dir,"hashedCache.Rdata",sep="");
	if( !file.exists(cache.file) ) return(NULL);

	hashS=lapply(hash.fS,function(f){f(X)});

	lock.dir=paste(cache.dir,"lock",sep="");
	while(!dir.create(lock.dir)) {cat("lock dir present, waiting\n"); system("sleep 1");} #dont try to read when someone is accessing
	load(cache.file); #hashedCacheS
	file.remove(lock.dir);

	#look for the current data in the cache
	SigmaStar=NULL;
	for( hc in hashedCacheS ){
		found.ds<-T;
		loaded.hashS<-hc$hashS;
		for( h.n in names(loaded.hashS) ){
			if( !all(hashS[[h.n]]==loaded.hashS[[h.n]]) ) {
				found.ds<-F; break;
			}
		}
		if( found.ds ) {
			cached.res<-hc[[paste(".",p.name,sep="")]];
			SigmaStar<-cached.res$SigmaStar;
			if( !is.null(SigmaStar) )
				cat("Found cached results for SigmaStar (", p.name,"), matching criteria {", paste(names(loaded.hashS),collapse=", "), "} ( produced on", cached.res$date, ")\n");
			break;
		}
	}
	return(SigmaStar);
}

put.SigmaStar.in.cache<-function(X,SigmaStar,p.name,cache.dir){
	if( is.null(cache.dir) ) return(NULL);
	if( !file.exists(cache.dir) ) dir.create(cache.dir);
	cache.file=paste(cache.dir,"hashedCache.Rdata",sep="");
	lock.dir=paste(cache.dir,"lock",sep="");

	hashS=lapply(hash.fS,function(f){f(X)});

	while(!dir.create(lock.dir)) {cat("lock dir present, waiting\n"); system("sleep 1");} #dont try to write when someone else is acessing
	on.exit(file.remove(lock.dir)); 

	if( !file.exists(cache.file) ) {
		hashedCacheS=list();
		found.ds<-F;
	} else {
		load(cache.file); #hashedCacheS

		#look for a place to put the data in the cache, the 
		for( hc.i in 1:length(hashedCacheS) ){
			hc<-hashedCacheS[[hc.i]];
			found.ds<-T;
			loaded.hashS<-hc$hashS;
			for( h.n in names(loaded.hashS) ){ #check if alla hashes match
				if( !all(hashS[[h.n]]==loaded.hashS[[h.n]]) ) {
					found.ds<-F; break;
				}
			}
			if( found.ds ) {
				if( !is.null(hc[[paste(".",p.name,sep="")]]$SigmaStar) ) {
					warning("WAME.put.SigmaStar.in.cache internal error: Data already in cache. Now stored twice.")
				}
				break;
			}
		}
	}
	if( !found.ds ) { #add a new entry if the dataset was not found
		hashedCacheS<-append(list(list(hashS=hashS)),hashedCacheS); hc.i=1;
	}
		
	new.elem<-list(list(SigmaStar=SigmaStar,date=date())); names(new.elem)<-paste(".",p.name,sep="");	
	hashedCacheS[[hc.i]]=append(new.elem,hashedCacheS[[hc.i]]);
	save(file=cache.file,list="hashedCacheS");
		lock.dir=paste(cache.dir,"lock",sep="");
}
