if( exists("WAME.manual") ) {
	library(KernSmooth);
}

dense.scatter<-function(data,xlim=range(data[,1]),ylim=range(data[,2]),nx=10,ny=10,do.plot=T,res,...){
	if( missing(res) ){
		x.range=xlim[2]-xlim[1];
		y.range=ylim[2]-ylim[1];
		
		x.step=x.range/nx;
		y.step=y.range/ny;
		
		x.I<-(data[,1]-xlim[1])%/%x.step+1;
		x.I[x.I==nx+1]=nx;

		y.I<-(data[,2]-ylim[1])%/%y.step+1;
		y.I[y.I==ny+1]=ny;
		
		res<-matrix(0,nrow=nx,ncol=ny);
		for( i in 1:nrow(data) ){
			res[x.I[i],y.I[i]]=res[x.I[i],y.I[i]]+1;
		}	
		
		bounds=list(x=seq(from=xlim[1], to=xlim[2], length=nx+1),y=seq(from=ylim[1], to=ylim[2], length=ny+1));

		res=list( hist=res, bounds=bounds );
	}
	
	if( do.plot ){
		image(x=res$bounds,z=res$hist>0,col=c(gray(1),gray(0)),...)
	}
	
	return(invisible(res));
}

cross.plot<-function(X,WAME.result=NULL,devS=list(dev),dev=default.pre.dev(),n.kde=100,use.dense.scatter=T,n.dense=n.kde,supplied.domain="x",subset.arrays,weights=NULL,Sigma=NULL,rel.bw=1/30){
	if(is.null(WAME.result)) { #what type of WAME-analysis has been performed?
		WAME.type=NA;
	} else if(class(WAME.result)=="WAME.paired.result") {
		WAME.type="paired";
	} else if(class(WAME.result)=="WAME.general.result") {
		WAME.type="general";
	} else {
		warning("cross.plot: The supplied WAME.result is not a result from WAME.paired or WAME.general. Skipping it.");
		WAME.type=NA;
	}

	if( !is.na(WAME.type) ) {
		if(WAME.type=="general") {
			needed.transformation=paste(tolower(supplied.domain),"2y",sep="");
			X=X%*%t(WAME.result$P[[needed.transformation]]);
			weights=WAME.result$weights$X;
			Sigma=WAME.result$Sigmas$Y;
		} else if(WAME.type=="paired") {
			weights=WAME.result$weights;
			Sigma=WAME.result$Sigma; 
		}
	}
	
	if( exists("weights") && is.vector(weights) ) {
		weights<-t(as.matrix(weights));
	}
	
	if( !missing(subset.arrays) ){
		X=X[,subset.arrays,drop=F];
		if(!is.null(weights)) weights<-weights[,subset.arrays,drop=F];
		if(!is.null(Sigma)) Sigma<-Sigma[subset.arrays,subset.arrays];
	}

	if( !is.null(Sigma) ) {
		Sigma.cor<-cov2Cor(Sigma);
	} else {
		Sigma.cor=null;
	}
	
	narrays<-ncol(X);
	
	plots.w<-plots.h<-max(5,narrays); #at least one inch per array

	gray.colors<-function(n){sapply(seq(from=1,to=0,length=n),gray)};
	#red.colors<-function(n,alpha=1){xs<-seq(from=1,to=0,length=n);rgb(1,xs,xs,alpha,max=1)};
	blackred.colors<-function(n,alpha=1){xs<-seq(from=1,to=0,length=n);rgb(xs,0,0,alpha,max=1)};
	
	d.range<-range(X); d.range<-c(d.range[1]-(d.range[2]-d.range[1])*0.1,d.range[2]+(d.range[2]-d.range[1])*0.1)
	d.mid<-mean(d.range);
	
	is2str<-function(x.i,y.i){paste(x.i,y.i,sep="_")};
	
	#pre-compute the two-dimensional kernel-density estimates to be used in all the devices
	imgs<-list();
	cat("Computing 2d-kde:\n");
	for( x.i in 1:(narrays-1) ) {
		for( y.i in (x.i+1):narrays ) {
			cat(".");
			#lims=rep(d.range,2);
			#img<-kde2d(X[,x.i],X[,y.i],lims=lims,h=rep((d.range[2]-d.range[1])*rel.bw*2,2),n=n.kde);
			img<-bkde2D(X[,c(x.i,y.i)],range=list(x=d.range,y=d.range),bandwidth=rep((d.range[2]-d.range[1])*rel.bw,2),gridsize=c(n.kde,n.kde));
			names(img)=c("x","y","z");			
			img$z<-img$z^(1/3);
			tmpList<-list(img); 			
			names(tmpList)<-is2str(x.i,y.i);
			imgs<-append(imgs, tmpList);
		}
		cat("\n");
	}
	
	#make the labels for the diagonal
	lab<-colnames(X);
	if( !is.null( weights ) ) {
		lab[lab!=""]<-paste(lab,"\n",sep="");
		lab<-paste(lab, "w:", apply(weights,2,function(ws){paste(round(ws,3),sep="/")}), sep="");
	}

	for( theDev in devS ) { 
		theDev$create(height=plots.h,width=plots.w);
		par(mar=rep(1,4));
		
		pairs(matrix(1:narrays,ncol=narrays), main=NA, labels=lab, xlim=d.range, ylim=d.range, label.pos=0.5,
			  upper.panel=function(x.i,y.i,...){
				  if( use.dense.scatter ) {
					dense.scatter(X[,c(x.i,y.i)],xlim=d.range,ylim=d.range,nx=n.dense,ny=n.dense,add=T);
				  } else {
					points( X[,x.i], X[,y.i], pch='.' );
				  }
				  #text(x=d.range[2],y=d.range[2],substitute(rho==tmp,list(tmp=round(Sigma.cor[x.i,y.i],2))),adj=c(1,1));
				  if(!is.null(Sigma.cor)) text(round(Sigma.cor[x.i,y.i],2),x=d.range[2],y=d.range[2],adj=c(1,1),col="blue");
			  },
			  lower.panel=function(x.i,y.i,...){
				  image(imgs[[is2str(x.i,y.i)]],add=T,col=c(gray.colors(256),rev(blackred.colors(256))));
				  #text(x=d.range[2],y=d.range[2],substitute(rho==tmp,list(tmp=round(Sigma.cor[x.i,y.i],2))),adj=c(1,1));
				  if(!is.null(Sigma.cor)) text(round(Sigma.cor[x.i,y.i],2),x=d.range[2],y=d.range[2],adj=c(1,1),col="blue");
			  },
			  diag.panel=function(i,...){
			      #text(lab[i],x=d.mid,y=d.mid,cex=1.5);
				  #text(substitute(sigma^2==tmp,list(tmp=round(Sigma[i,i],3))),x=d.range[2],y=d.range[2],adj=c(1,1));
				  if(!is.null(Sigma)) text(round(Sigma[i,i],3),x=d.range[2],y=d.range[2],adj=c(1,1),col="blue");
			  }
		);
		theDev$close();
	}
}

pairs.WAME.general.result<-function(x,...){cross.plot(WAME.results=x,...)};
pairs.WAME.paired.result<-function(x,...){cross.plot(WAME.results=x,...)};