Id<-function(n){diag(rep(1,n))}

realZero<-function(M,tol=10^-10){
	M[abs(M)<=tol]<-0; M
}

# E: returns a base matrix for the space spanned through the design matrix D by the parameter-space spanned by C 
E<-function(D,C=Id(ncol(D)),Sigma=Id(nrow(D)),iSigma=ginv(Sigma)){ 
	D%*%ginv(t(D)%*%iSigma%*%D)%*%t(C)
}

# P: returns a projection matrix for the subspace with base E
P<-function(Em=E(...),Sigma=Id(nrow(Em)),iSigma=ginv(Sigma),...){
	Em%*%ginv(t(Em)%*%iSigma%*%Em)%*%t(Em)%*%iSigma
}

# Pc: returns a projection matrix for the complement to the subspace with base E
Pc<-function(...){
	P1<-P(...);
	Id(ncol(P1))-P1;
}

# ParEstP : "Parameter Estimates in Projection"
#           The weights of the row-vectors in Em, answering to the projection onto the space spanned by Em
ParEstP<-function(Em=E(...),Sigma=Id(nrow(Em)),iSigma=ginv(Sigma),...){
	ginv(t(Em)%*%iSigma%*%Em)%*%t(Em)%*%iSigma;
}

norm.sq<-function(X,Sigma){
	if( !is.matrix(X) ){
		X<-t(as.matrix(X))
	}
	if(missing(Sigma)) Sigma<-Id(ncol(X));
	iSigma<-ginv(Sigma);
	
	#norm.sq(X,Sigma)=
	#apply(X,1,function(x.g){x.g%*%iSigma%*%x.g})=
	#apply(X,1,function(x.g){(x.g%*%mroot(iSigma))%*%(mroot(iSigma)%*%x.g)})=
	#apply(X,1,function(x.g){sum((x.g%*%mroot(iSigma))^2)})=
	#rowSums((x.g%*%mroot(iSigma))^2)
	res<-as.vector(rowSums((X%*%mroot(iSigma))^2));
	names(res)<-rownames(X);
	res;
}

projection.set<-function(D,C,C0,Sigma=Id(nrow(D))){
	if( !missing(C)&&!is.matrix(C) ){
		C<-t(as.matrix(C))
	}

	n<-ncol(Sigma);
	iSigma=ginv(Sigma);
	E.V=E(D=D,iSigma=iSigma);
	P.V=P(E=E.V,iSigma=iSigma);
	P.Vc=Id(n)-P.V;
	if( !missing(C0) ) {
		E.V0=E(D=D,C=C0,iSigma=iSigma);	E.VC=NULL;
		P.V0=P(E=E.V0,iSigma=iSigma);
		P.VC=P.V-P.V0;
		C=NULL;
	} else {
		E.VC=E(D=D,C=C,iSigma=iSigma); E.V0=NULL;
		P.VC=P(E=E.VC,iSigma=iSigma);
		P.V0=P.V-P.VC;
		C0=NULL;
	}
	P.V0c=Id(n)-P.V0;
	return(list(C=C,C0=C0,D=D,Sigma=Sigma,iSigma=iSigma,E.V=E.V,E.VC=E.VC,E.V0=E.V0,P.V=P.V,P.Vc=P.Vc,P.VC=P.VC,P.V0=P.V0,P.V0c=P.V0c))
}

WAME.projections<-function(X,D,C,C0,wSigma.X=Id(nrow(D)),array.names=colnames(X)) {
	narrays<-nrow(D);

	pset<-projection.set(D=D,C=C,C0=C0,Sigma=wSigma.X);
	
	P.x2y<-pset$P.V0c; dimnames(P.x2y)<-list(array.names,array.names);
	k<-computeRank(P.x2y);
	P.y2z<-matrix(0,nrow=k,ncol=narrays); colnames(P.y2z)<-array.names;
	P.z2y<-matrix(0,nrow=narrays,ncol=k); rownames(P.z2y)<-array.names;
	kept<-rep(T,narrays);
	for( i in 1:narrays ) {
		base<-P.x2y[kept&(i>(1:narrays)),,drop=F];
		this.row<-P.x2y[i,];
		nb<-nrow(base);
		if( computeRank(rbind(base,this.row))<=nb ){ #rank hasnt increased with the new row
			P.z2y[i,1:nb]<-realZero(ginv(base%*%t(base))%*%base%*%this.row);
			kept[i]<-F;
		} else {
			P.z2y[i,nb+1]<-1;
		}
	}
	colnames(P.z2y)<-rownames(P.y2z)<-array.names[kept];
	diag(P.y2z[,kept])<-1;
	P.x2z<-realZero(P.y2z%*%P.x2y);
	
	return( list(x2z=P.x2z,x2y=P.x2y,y2z=P.y2z,z2y=P.z2y) );
}
